//
//  GetSocialNotificationContent.h
//  GetSocial
//
//  Created by Orest Savchak on 9/18/18.
//  Copyright © 2018 GetSocial BV. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <GetSocial/GetSocialConstants.h>

NS_ASSUME_NONNULL_BEGIN

@interface GetSocialNotificationContent : NSObject

- (instancetype)init NS_UNAVAILABLE;

/*!
 * Create notification with text.
 * @param text text to be displayed to receivers.
 * @return new notification content.
 */
+ (GetSocialNotificationContent *)withText:(NSString *)text;

/*!
 * Create notification from the template configured on the GetSocial Dashboard.
 * @param templateName name of the template on the GetSocial Dashboard. Case-sensitive.
 * @return new notification content.
 */
+ (GetSocialNotificationContent *)withTemplateName:(NSString *)templateName;

/*!
 * Set notification action to be performed on click.
 * @param actionType what should be done when user clicks on notification
 */
- (void)setActionType:(GetSocialNotificationActionType)actionType;

/*!
 * Add action data you could retrieve on the receiver side. Also you could specify some additional data for default actions.
 * Use one GetSocialNotificationKey for default actions or pass your custom NSString *.
 * @param value action value.
 * @param key action key.
 */
- (void)addActionDataValue:(NSString *)value forKey:(NSString *)key;

/*!
 * Add all keys and values from map to action data.
 * @param templateData map of action keys and values.
 */
- (void)addActionData:(NSDictionary<NSString *, NSString *> *)templateData;

/*!
 * Set notification text. If you use template, your text will be overriden by this.
 * @param text notification text.
 */
- (void)setText:(NSString *)text;

/*!
 * Set notification title. If you use template, your title will be overriden by this.
 * @param title notification title.
 */
- (void)setTitle:(NSString *)title;

/*!
 * Set template name. Notification will use values from the GetSocial Dashboard as title and text.
 * But [GetSocialNotificationContent setTitle:] or [GetSocialNotificationContent setText:] have higher priority and will override template values.
 * @param templateName name of the template on the dashboard. Case-sensitive.
 */
- (void)setTemplateName:(NSString *)templateName;

/*!
 * If you specified placeholders on the GetSocial Dashboard for your template title or text - you can replace it using this method.
 * For example, if your template text it "Hello,  [USERNAME].", call [notificationContent addTemplatePlaceholderValue:@"My actual name"
 * withKey:@"USERNAME"]. Brackets should be omitted in key. Won't make any effect without template name.
 * @param value actual text that should be used instead.
 * @param key placeholder on the GetSocial Dashboard.
 */
- (void)addTemplatePlaceholderValue:(NSString *)value forKey:(NSString *)key;

/*!
 * Add all keys and values from map to template placeholders.
 * @param templateData template placeholders map.
 */
- (void)addTemplatePlaceholders:(NSDictionary<NSString *, NSString *> *)templateData;

@end

NS_ASSUME_NONNULL_END
